% LTFAT - Quadratic time-frequency distributions
%
%  Jordy van Velthoven, 2014 - 2018.
%
%  Quadratic distributions
%    AMBIGUITYFUNCTION   -  Ambiguity function
%    WIGNERVILLEDIST     -  Wigner-Ville distribution
%    DRIHACZEKDIST       -  Discrete Rihaczek distribution
%    QUADTFDIST          -  Generic Quadratic distribution
%
%  Plots
%    PLOTQUADTFDIST      - Plot quadratic time-frequency distribution
% 
%  For help, bug reports, suggestions etc. please visit 
%  http://github.com/ltfat/ltfat/issues
%
%   Url: http://ltfat.github.io/doc/quadratic/Contents.html

% Copyright (C) 2005-2018 Peter L. Soendergaard <peter@sonderport.dk> and others.
% This file is part of LTFAT version 2.4.0
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

