% LTFAT - Signals
%
%  Signal generators
%    CTESTFUN       -  Complex valued test function.
%    NOISE          -  Stochastic noise generator.
%    PINKNOISE      -  Pink noise.
%    EXPCHIRP       -  Exponential chirp.
%
%  Sound signals.
%    BAT            -  Bat chirp.
%    BATMASK        -  Mask for bat signal.
%    GREASY         -  Woman speaking the word 'greasy'
%    COCKTAILPARTY  -  Sentence by male, native English speaker.
%    GSPI           -  Glockenspiel test signal.
%    LINUS          -  Linus pronouncing Linux.
%    LTFATLOGO      -  Synthetic sound from spectrogram reconstruction.
%    OTOCLICK       -  Click-evoked otoacoustic emmision. 
%    TRAINDOPPLER   -  Sound of passing train.
%
%  Images.
%    CAMERAMAN      -  Greyscale image of the cameraman.
%    LICHTENSTEIN   -  Color image of the Lichtenstein castle.
%    LTFATTEXT      -  Black and white word: 'LTFAT'.
%
%  For help, bug reports, suggestions etc. please visit 
%  http://github.com/ltfat/ltfat/issues
%
%
%   Url: http://ltfat.github.io/doc/signals/Contents.html

% Copyright (C) 2005-2018 Peter L. Soendergaard <peter@sonderport.dk> and others.
% This file is part of LTFAT version 2.4.0
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


