% LTFAT - Base routines
%
%  Peter L. Soendergaard, 2009 - 2018.
%
%  Basic routines
%    LTFATSTART       -  Start the toolbox
%    LTFATSTOP        -  Stop the toolbox
%    LTFATHELP        -  Help
%    LTFATMEX         -  Compile Mex/Oct interfaces
%    LTFATBASEPATH    -  Return the base path
%    ISOCTAVE         -  True if interpreter is Octave
%
%  Parameter handling
%    LTFATARGHELPER   -  Handle optional parameters of functions
%    LTFATGETDEFAULTS -  Get the default values for a function
%    LTFATSETDEFAULTS -  Get the default values for a function
%    SCALARDISTRIBUTE -  Expand parameters to common size
%
%  Graphical user interfaces
%    MULACLAB         -  Short Time-Fourier transform modification in Matlab
%
%  For help, bug reports, suggestions etc. please visit 
%  http://github.com/ltfat/ltfat/issues
%
%   Url: http://ltfat.github.io/doc/Contents.html

% Copyright (C) 2005-2018 Peter L. Soendergaard <peter@sonderport.dk> and others.
% This file is part of LTFAT version 2.4.0
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

